#!/usr/bin/env python
#-*- coding:utf-8 -*-

#-----------------------------------------------
#import modules
#-----------------------------------------------

from __future__ import print_function
import os,sys
import netCDF4
from scipy import arcsin,cos,sin,degrees,radians,arctan2,pi
import scipy
import datetime

#-----------------------------------------------
# function read_var
#-----------------------------------------------

def read_var(filename,variables):
    """ Extract variables from a netCDF

        return a list of netCDF4 array but time as int from seconds since ordinal
        variables : list of string

    """
    if type(variables) is not list:
        variables=[variables]

    if type(filename) is not list:
        try:
            ncfile=netCDF4.Dataset(filename)
        except Exception as err:
            sys.stderr.write('{0}:{1}\n'.format(err,filename))
    else:
        ncfile=netCDF4.MFDataset(filename)

    vararray=[]
    for var in variables:
        if var != 'time':
            try:
                vardata=scipy.array(ncfile.variables[var])
            except Exception as err:
                sys.stderr.write('{0} not found in {1}\n'.format(err,filename))
                sys.exit()
        # special treatment for time
        else:
            vardata=ncfile.variables[var]
            try:
                units=vardata.units.split()
               # for times units with hh/mm/ss since date
                if len(units)>3:
                    # if units is based on matlab time (0000-01-02) (watersip)
                    if units[2].split('-')[0] == '0000':
                        sdate='0001-{0}-{1}'.format(units[2].split('-')[1],units[2].split('-')[2])
                        offset=-368
                    else:
                        sdate=units[2]
                        offset=0
                    try:
                        date=datetime.datetime.strptime(sdate+" "+units[3],"%Y-%m-%d %H:%M:%S")
                    except:
                        date=datetime.datetime.strptime(sdate+" "+units[3],"%Y-%m-%d %H:%M")
                    # seconds since units
                    if units[0] == "seconds":
                        vardata=[date+datetime.timedelta(seconds=int(time)+offset) for time in vardata[:]]
                        
                    elif units[0] == "hours":
                        vardata=[date+datetime.timedelta(hours=int(time)+offset) for time in vardata[:]]
                    # for days since units
                    else:
                        vardata=[date+datetime.timedelta(days=time+offset) for time in vardata[:]]
                # for time units without hh/mm/ss since date
                else:
                    date=datetime.datetime.strptime(units[2],"%Y-%m-%d")
                    vardata=[date+datetime.timedelta(days=int(time)) for time in vardata[:]]
            except AttributeError:  
                vardata=scipy.array(vardata)
                
        if var in ['QV','QC','QI','QR','QG','QS']:
            if vardata.max()<1:
                vardata=vardata*1000
        elif var in ['T','T_2M']:
            if vardata.max()>100:
                vardata=vardata-273.15
        elif var in ['pressure']:
            if vardata.max()>10000:
                vardata=vardata/100
        vararray.append(vardata)
    
    ncfile.close()
    return vararray

#-----------------------------------------------
# function read_gattributes
#-----------------------------------------------

def read_gattributes(filename):
	""" Read global attributes from a netCDF"""
	gattributes=netCDF4.Dataset(filename).__dict__
	return gattributes

#-----------------------------------------------
# function read_dimensions
#-----------------------------------------------
def read_dimensions(filename):
    """ Read dimensions"""
    dimensions=netCDF4.Dataset(filename).dimensions
    return dimensions

#-----------------------------------------------
# function read_dimensions
#-----------------------------------------------
def read_variables(filename):
    """ Read variables"""
    variables=list(netCDF4.Dataset(filename).variables.keys())
    return variables

#-----------------------------------------------
# function create
#-----------------------------------------------

def create(outname,dimensions,gattributes):
    """ create a netCDF """
    ncfile=netCDF4.Dataset(outname, 'w', format='NETCDF3_CLASSIC')
    for dim in dimensions:
        d=ncfile.createDimension(dim[0],dim[1])
    ncfile.setncatts(gattributes)
    return ncfile

#-----------------------------------------------
# function addvar
#-----------------------------------------------
def addvar(ncfile,varname,vardata,dimensions):
    """ Add a variable to a netcdf"""
    var = ncfile.createVariable(varname,vardata.dtype,dimensions)
    try:
        var[:] = vardata
    except Exception as e :
        sys.stderr.write('{0} in addvar()\n'.format(e))
        sys.exit(1)

#-----------------------------------------------
# function getdelta
#-----------------------------------------------
def getdelta(ncfile):
    """ return dlon,dlat from netCDF """
    rlon=netCDF4.Dataset(ncfile).variables['rlon']
    rlat=netCDF4.Dataset(ncfile).variables['rlon']
    dlon=rlon[1]-rlon[0]
    dlat=rlat[1]-rlat[0]

    return dlon,dlat

#-----------------------------------------------
# function rotcoo
#-----------------------------------------------

def rotcoo(longs,lats,direction,lonpol=None,latpol=None,ncfile=None,view=False):
    """ Rotated coordinates from non-rotated system to rotated and viceversa """
    
    if lonpol is None:
        if ncfile is not None:
            lonpol=netCDF4.Dataset(ncfile).variables['rotated_pole'].grid_north_pole_longitude
        else:
            sys.exit("netCDF file is needed")
    if latpol is None:
        if ncfile is not None:
            latpol=netCDF4.Dataset(ncfile).variables['rotated_pole'].grid_north_pole_latitude
        else:
            sys.exit("netCDF file is needed")

    try:
        scipy.putmask(longs,longs > 180, longs-180)
    except:
        longs=scipy.array([longs])
        lats=scipy.array([lats])
        scipy.putmask(longs,longs > 180, longs-180)
    #prepare for masking arrays - 'conventional' arrays won't do it
    longs = scipy.ma.array(longs)
    lats = scipy.ma.array(lats)
    #mask values below a certain threshold
    longs_masked = scipy.ma.masked_where(longs < -998 , longs)
    lats_masked = scipy.ma.masked_where(lats < -998 , lats)
    
    # transform in radians

    zlongs=radians(longs_masked)
    zlats=radians(lats_masked)
    zlonpol=radians(lonpol)
    zlatpol=radians(latpol)

    # some needs variables
    zcoslatpol=cos(zlatpol)
    zsinlatpol=sin(zlatpol)

    if direction == 'n2r':
        # new lat (rlat)
        arg1=sin(zlats)*zsinlatpol
        arg2=cos(zlats)*zcoslatpol*cos(zlongs-zlonpol)
        newlat=degrees(arcsin(arg1+arg2))
        # new lon (rlon)
        arg1=-sin(zlongs-zlonpol)*cos(zlats)
        arg2=-zsinlatpol*cos(zlats)*cos(zlongs-zlonpol)+zcoslatpol*sin(zlats)
        newlon=degrees(arctan2(arg1,arg2))
    elif direction == 'r2n':
        # new lat
        arg1=zcoslatpol*cos(zlats)*cos(zlongs)+zsinlatpol*sin(zlats)
        newlat=degrees(arcsin(arg1))
        # new lon 
        arg1=sin(zlonpol)*(-zsinlatpol*cos(zlongs)*cos(zlats)+zcoslatpol*sin(zlats))-cos(zlonpol)*sin(zlongs)*cos(zlats)
        arg2=cos(zlonpol)*(-zsinlatpol*cos(zlongs)*cos(zlats)+zcoslatpol*sin(zlats))+sin(zlonpol)*sin(zlongs)*cos(zlats)
        newlon=degrees(arctan2(arg1,arg2))

    if view:
        print('{0:1.4f} {1:1.4f}'.format(newlon[0],newlat[0]))
    return newlon,newlat

if __name__ == '__main__':
    a=rotcoo(scipy.array([[-2.045,-2.04],[-2.045,-2.04]]),scipy.array([[0.039,0.02],[0.039,0.02]]),'r2n',ncfile='../../../essai/lfff20111010_0200p.nc')
    print(a[0])
    print(a[1])

