#!/bin/csh

# -----------------------------------------------------------------------------
# Set some general parameters
# -----------------------------------------------------------------------------

# Usage
if ( $#argv == 0 ) then
  echo "install.sh [lib|core|goodies|links|all|docu|clean|tag"
  exit 0
endif

# Set the mode
set mode = $1

# Set path to SVN repository
set svnpath=https://svn.iac.ethz.ch/pub/lagranto.cosmo/

# Set path for development
set path_devel = "${DYN_TOOLS}/lagranto.cosmo/"
set path_sync  = "${DYN_TOOLS}/lagranto/"

# Init Fortran compiler and set netCDF acccordingly
setenv FORTRAN pgf90

# Init netCDF library depending on the Fortran compiler
if ( "${FORTRAN}" == "pgf90" ) then
  module purge
  module load dyn_tools
  # module load netcdf/4.3.2-pgf90
  module list 

else if ( "${FORTRAN}" == "gfortran" ) then
  module purge
  module load gfortran
  module load netcdf/4.1.1

else if ( "${FORTRAN}" == "ifort" ) then
  module purge
  module load ifort/10.1.017
  module load netcdf/4.1.1-ifort

else
  echo "Fortran Compiler ${FORTRAN} not supported... Exit"
  exit 1

endif

# Set Lagranto path
setenv LAGRANTO ${path_devel}

# -----------------------------------------------------------------------------
# Create a new tag in SVN repository
# -----------------------------------------------------------------------------

if ( "${mode}" == "tag" ) then
   svn info
   if ( "${#argv}" != 2 ) then
     echo "Usage: install.csh tag id <id=tag number>"
   else
     set tagnr = $2
   endif
   svn copy ${svnpath}/trunk ${svnpath}/tags/${tagnr} -m "Release ${tagnr}"
   exit 0
endif

# -----------------------------------------------------------------------------
# Set internal parameters and detailed installation mode
# -----------------------------------------------------------------------------

# Set netCDF paths
setenv NETCDF_LIB `nc-config --flibs`
setenv NETCDF_INC `nc-config --fflags`

# Set list of core programs
set core  = "create_startf caltra trace select density lidar" 

# Set list of goodies
set tools = "changet extract gettidiff getvars list2lsl lsl2list mergetra newtime reformat timeres trainfo difference datelist geo2rot rot2geo tracal lsl2rdf" 

# Set list of libraries
set libs  = "iotra ioinp inter times libcdfioplus_cosmo rotate"

# Core programs
foreach prog ( $core )
   if ( "${prog}" == "${mode}" ) then
      set core  = ${prog}
      set mode  = "core"
   endif
end

# Goodies
foreach tool ( $tools )
   if ( "${tool}" == "${mode}" ) then
      set tools = ${tool}
      set mode  = "goodies"
   endif
end

# Libraries
foreach lib ( $libs )
   if ( "${lib}" == "${mode}" ) then
      set libs  = ${lib}
      set mode  = "lib"
   endif
end

# Check that the mode is ok 
if ( "${mode}" == "all"     ) goto modeok
if ( "${mode}" == "lib"     ) goto modeok
if ( "${mode}" == "core"    ) goto modeok
if ( "${mode}" == "goodies" ) goto modeok
if ( "${mode}" == "links"   ) goto modeok
if ( "${mode}" == "clean"   ) goto modeok
if ( "${mode}" == "docu"    ) goto modeok
if ( "${mode}" == "sync"    ) goto modeok
echo "Unsupported mode ${mode} ... Stop"
exit 1

modeok:

# -----------------------------------------------------------------------------
# Make clean 
# -----------------------------------------------------------------------------

if ( "${mode}" == "clean" ) then

cd ${LAGRANTO}/

foreach prog ( $core )
   \rm -f ${prog}/${prog} ${prog}/${prog}.o
end
\rm -f trace/calvar.o select/special.o

foreach tool ( $tools )
  \rm -f goodies/${tool} goodies/${tool}.o 
end
\rm -f goodies/*.mod goodies/*.o

\rm lib/*.a lib/*.o

foreach prog ( $core )
   \rm -f bin/${prog} bin/${prog}.sh  bin/${prog}.cosmo
end
\rm -f bin/seltra bin/seltra.sh bin/seltra.cosmo
foreach tool ( $tools )
  \rm -f bin/${tool} bin/${tool}.sh bin/${tool}.cosmo
end
\rm -f bin/lagrantohelp.sh bin/lagrantohelp.cosmo
\rm -f bin/startf bin/startf.sh bin/startf.cosmo
\rm -f bin/lagranto.sh bin/lagranto.cosmo

\rm ${LAGRANTO}/startf

exit 0

endif

# -----------------------------------------------------------------------------
# Install reference 
# -----------------------------------------------------------------------------

if ( ("${mode}" == "docu") | ("${mode}" == "all" ) ) then
    
echo "-----------------------------------------------------------------"
echo "Installing documentation"
echo "-----------------------------------------------------------------"
echo

cd ${LAGRANTO}/docu/reference/

\rm -f reference.ps
\rm -f reference2.ps
groff -man ../man/*.0 > reference2.ps
ps2pdf reference2.ps

latex title
dvips title
ps2pdf title.ps

gs -dBATCH -dNOPAUSE -q -sDEVICE=pdfwrite -sOutputFile=reference.pdf title.pdf reference2.pdf

\rm -f *.aux *.log *.dvi
\rm -f title.ps  reference2.ps
\rm -f title.pdf reference2.pdf

endif

if ( "${mode}" == "docu" ) exit 0

# -----------------------------------------------------------------------------
# Install libraries 
# -----------------------------------------------------------------------------

if ( ("${mode}" == "lib") | ("${mode}" == "all" ) ) then

echo "-----------------------------------------------------------------"
echo "Installing libraries"
echo "-----------------------------------------------------------------"
echo

# Change to library directory
cd ${LAGRANTO}/lib

# Loop over all libraries - compile and make library
foreach lib ( $libs )

if ( "${lib}" == "libcdfioplus_cosmo" ) then
    \rm -f libcdfioplus_cosmo.f
    cat libcdfioplus_cosmo_mch.f libcdfioplus_cosmo_cf.f libcdfioplus_cosmo_ive.f libcdfioplus_cosmo_master.f > libcdfioplus_cosmo.f
endif

\rm -f ${lib}.a
\rm -f ${lib}.o
if ( -f ${lib}.f ) then
  echo ${FORTRAN} -c -O ${lib}.f
  ${FORTRAN} -c -O ${NETCDF_INC} ${lib}.f
endif
if ( -f ${lib}.f90 ) then
  echo ${FORTRAN} -c -O ${lib}.f90
  ${FORTRAN} -c -O ${NETCDF_INC} ${lib}.f90
endif
ar r ${lib}.a ${lib}.o
\rm -f ${lib}.l ${lib}.o
ranlib ${lib}.a
if ( ! -f ${lib}.a ) then
  echo "Problem in compiling ${lib} ... Stop"
  exit 1
endif

end

endif

if ( "${mode}" == "lib" ) exit 0

# -----------------------------------------------------------------------------
# Check that libraries are ok
# -----------------------------------------------------------------------------

echo
echo "-----------------------------------------------------------------"
echo "Check that all libraries are available"
echo "-----------------------------------------------------------------"
echo

# Change to library directory
cd ${LAGRANTO}/lib

# Check whether all libraries are available
foreach lib ( $libs )

if ( ! -f ${lib}.a ) then
  echo "Library ${lib} missing... Stop"
  exit 1
else
  ls -l ${lib}.a    
endif

end

# Exit if only libraries shoudl be installed
if ( "${mode}" == "lib" ) exit 0

# -----------------------------------------------------------------------------
# Compile Lagrango core programs
# -----------------------------------------------------------------------------

if ( ("${mode}" == "all" ) | ("${mode}" == "core" ) ) then

echo
echo "-----------------------------------------------------------------"
echo "Installing Lagranto core programs"
echo "-----------------------------------------------------------------"

foreach prog ( $core )

echo
echo "----- $prog"
echo
cd ${LAGRANTO}/${prog}
\rm -f ${prog}.o 
\rm -f ${prog}
if ( "${prog}" == "trace"  ) \rm calvar.o

if ( -f ${prog}.make ) then
   make -f ${prog}.make
else if ( -f ${prog}.install ) then
   ./${prog}.install
endif

if ( ! -f ${prog} ) then
  echo "Problem in compiling ${prog} ... Stop"
  exit 1
endif

end

endif

if ( "${mode}" == "core" ) exit 0

# -----------------------------------------------------------------------------
# Check that all Lagranto core programs are available
# -----------------------------------------------------------------------------

echo
echo "-----------------------------------------------------------------"
echo "Check that all Lagranto core programs are available"
echo "-----------------------------------------------------------------"
echo

foreach prog ( $core )

  cd ${LAGRANTO}/${prog}
  if ( ! -f ${prog} ) then
    echo "${prog} is missing... Stop"
    exit 1
  else
    ls -l ${prog}    
  endif

end

# Exit if only core programs shoudl be installed
if ( "${mode}" == "core" ) exit 0

# -----------------------------------------------------------------------------
# Compile Lagrango goodies
# -----------------------------------------------------------------------------

if ( ("${mode}" == "all" ) | ("${mode}" == "goodies" ) ) then

echo
echo "-----------------------------------------------------------------"
echo "Installing Lagranto goodies"
echo "-----------------------------------------------------------------"

cd ${LAGRANTO}/goodies

foreach tool ( $tools )

echo
echo "----- ${tool}"
echo
\rm -f ${tool}.o
\rm -f ${tool}

if ( -f ${tool}.make ) then
   make -f ${tool}.make
else if ( -f ${tool}.install ) then
   ./${tool}.install
endif

if ( ! -f ${tool} ) then
  echo "Problem in compiling ${tool} ... Stop"
  exit 1
endif

end

endif

if ( "${mode}" == "goodies" ) exit 0

# -----------------------------------------------------------------------------
# Check that all Lagranto goodies are available
# -----------------------------------------------------------------------------

echo
echo "-----------------------------------------------------------------"
echo "Check that all Lagranto goodies are available"
echo "-----------------------------------------------------------------"
echo

cd ${LAGRANTO}/goodies

foreach tool ( $tools )

if ( ! -f ${tool} ) then
  echo "${tool} is missing... Stop"
  exit 1
else
  ls -l ${tool} 
endif

end

endif

# Exit if only goodies should be installed
if ( "${mode}" == "goodies" ) exit 0

# -----------------------------------------------------------------------------
# Create links to programs
# -----------------------------------------------------------------------------

if ( ("${mode}" == "all" ) | ("${mode}" == "links" )  ) then

echo
echo "-----------------------------------------------------------------"
echo "Create links in ${LAGRANTO}/bin/"
echo "-----------------------------------------------------------------"
echo

cd ${LAGRANTO}/bin

ln -svf ${LAGRANTO}/bin/lagranto                   lagranto.cosmo  
ln -svf ${LAGRANTO}/bin/lagrantohelp               lagrantohelp.cosmo
ln -svf ${LAGRANTO}/caltra/caltra.sh               caltra.cosmo
ln -svf ${LAGRANTO}/create_startf/create_startf.sh create_startf.cosmo
ln -svf ${LAGRANTO}/select/select.sh               select.cosmo
ln -svf ${LAGRANTO}/select/select.sh               seltra.cosmo
ln -svf ${LAGRANTO}/trace/trace.sh                 trace.cosmo
ln -svf ${LAGRANTO}/density/density.sh             density.cosmo
ln -svf ${LAGRANTO}/create_startf/create_startf.sh startf.cosmo
ln -svf ${LAGRANTO}/lidar/lidar.sh                 lidar.cosmo

ln -svf ${LAGRANTO}/bin/lagranto                   lagranto
ln -svf ${LAGRANTO}/bin/lagrantohelp               lagrantohelp
ln -svf ${LAGRANTO}/caltra/caltra.sh               caltra
ln -svf ${LAGRANTO}/create_startf/create_startf.sh create_startf
ln -svf ${LAGRANTO}/select/select.sh               select
ln -svf ${LAGRANTO}/trace/trace.sh                 trace
ln -svf ${LAGRANTO}/density/density.sh             density
ln -svf ${LAGRANTO}/create_startf/create_startf.sh startf
ln -svf ${LAGRANTO}/lidar/lidar.sh                 lidar

ln -svf ${LAGRANTO}/bin/lagranto                   lagranto.sh
ln -svf ${LAGRANTO}/bin/lagrantohelp               lagrantohelp.sh
ln -svf ${LAGRANTO}/caltra/caltra.sh               caltra.sh
ln -svf ${LAGRANTO}/create_startf/create_startf.sh create_startf.sh
ln -svf ${LAGRANTO}/select/select.sh               select.sh
ln -svf ${LAGRANTO}/trace/trace.sh                 trace.sh
ln -svf ${LAGRANTO}/density/density.sh             density.sh
ln -svf ${LAGRANTO}/create_startf/create_startf.sh startf.sh
ln -svf ${LAGRANTO}/lidar/lidar.sh                 lidar.sh

foreach tool ( $tools )

ln -svf ${LAGRANTO}/goodies/${tool}.sh     ${tool}
ln -svf ${LAGRANTO}/goodies/${tool}.sh     ${tool}.cosmo
ln -svf ${LAGRANTO}/goodies/${tool}.sh     ${tool}.sh

end

# Set extra name for <select> to avoid conflict in BASH
ln -svf ${LAGRANTO}/select/select.sh        seltra
ln -svf ${LAGRANTO}/select/select.sh        seltra.sh
ln -svf ${LAGRANTO}/select/select.sh        seltra.cosmo

endif

# Set link startf -> create_startf
cd ${LAGRANTO}
\rm -f startf
ln -svf ${LAGRANTO}/create_startf startf

# -----------------------------------------------------------------------------
# Synchronise ( development -> operational ) 
# -----------------------------------------------------------------------------

if ( ("${mode}" == "all" ) | ("${mode}" == "sync" )  ) then

echo
echo "-----------------------------------------------------------------"
echo "Sync ( lagranto.cosmo -> lagranto )"
echo "-----------------------------------------------------------------"
echo

cd ${path_sync}/bin/

ln -svf ${path_devel}/bin/lagranto.sh            lagranto.cosmo
ln -svf ${path_devel}/bin/lagrantohelp.sh        lagrantohelp.cosmo

ln -svf ${path_devel}/caltra/caltra.sh           caltra.cosmo
ln -svf ${path_devel}/startf/create_startf.sh    create_startf.cosmo
ln -svf ${path_devel}/select/select.sh           select.cosmo
ln -svf ${path_devel}/trace/trace.sh             trace.cosmo
ln -svf ${path_devel}/density/density.sh         density.cosmo
ln -svf ${path_devel}/startf/create_startf.sh    startf.cosmo
ln -svf ${path_devel}/lidar/lidar.sh             lidar.cosmo
ln -svf ${path_devel}/lidar/seltra.sh            seltra.cosmo

foreach tool ( $tools )

ln -svf ${path_devel}/goodies/${tool}.sh      ${tool}.cosmo

end

# Set link startf -> create_startf
cd ${LAGRANTO}
\rm -f startf
ln -svf ${LAGRANTO}/create_startf startf
 
# Set all permissions
chmod -R og+rx ${path_sync}/bin/

endif

# -----------------------------------------------------------------------------
# Final tasks
# -----------------------------------------------------------------------------

echo
echo "-----------------------------------------------------------------"
echo "Installation complete"
echo "-----------------------------------------------------------------"
echo
echo "Please set the environmental variable LAGRANTO"
echo
echo "    setenv LAGRANTO ${LAGRANTO}"
echo
 




