c     ************************************************************
c     * This package provides input routines to read the wind    *
c     * and other fields from IVE necdf files. The routines are  *
c     *                                                          *
c     * 1) input_open  : to open a data file                     *
c     * 2) input_grid  : to read the grid information, including *
c     *                  the vertical levels                     *
c     * 3) input_wind  : to read the wind components             *
c     * 4) input_close : to close an input file                  *
c     *                                                          *
c     * The file is characterised by an filename <filename> and  *
c     * a file identifier <fid>. The horizontal grid is given by *
c     * <xmin,xmax,ymin,ymax,dx,dy,nx,ny> where the pole of the  *
c     * rotated grid is given by <pollon,pollat>. The vertical   *
c     * grid is characterised by the surface height <zb> and     *
c     * the model level height (given on the W grid). The number *
c     * of levels is given by <nz>. Finally, the retrieval of the* 
c     * wind <field> with name <fieldname>  is characterised by  *
c     a <time> and a missing data value <mdv>.                   *
c     *                                                          *
c     * Author: Michael Sprenger, Spring 2011/2012               *
c     ************************************************************

c     ------------------------------------------------------------
c     Open input file
c     ------------------------------------------------------------

      subroutine input_open (fid,filename)

c     Open the input file with filename <filename> and return the
c     file identifier <fid> for further reference. 

      implicit none

c     Declaration of subroutine parameters
      integer      fid              ! File identifier
      character*80 filename         ! Filename

c     Declaration of auxiliary variables
      integer      ierr

c     Open netcdf file
      call cdfopn(filename,fid,ierr)
      if (ierr.ne.0) goto 900
      
c     Exception handling
      return
      
 900  print*,'Cannot open input file ',trim(filename)
      stop

      end


c     ------------------------------------------------------------
c     Read information about the grid
c     ------------------------------------------------------------
      
      subroutine input_grid 
     >             (fid,fieldname,xmin,xmax,ymin,ymax,dx,dy,nx,ny,
     >              time,pollon,pollat,polgam,z3,zb,nz,stagz,timecheck)

c     Read grid information at <time> from file with identifier <fid>. 
c     The horizontal grid is characterized by <xmin,xmax,ymin,ymax,dx,dy>
c     with pole position at <pollon,pollat> and grid dimension <nx,ny>.
c     The 3d arrays <z3(nx,ny,nz)> gives the vertical coordinates, either
c     on the staggered or unstaggered grid (with <stagz> as the flag).
c     The surface height is given in <zb(nx,ny)>. If <fid> is negative, 
c     only the grid dimensions and grid parameters (xmin...pollat,nz) are 
c     determined and returned (this is needed for dynamical allocation of 
c     memory).

      implicit none

c     Declaration of subroutine parameters 
      integer      fid                  ! File identifier
      real         xmin,xmax,ymin,ymax  ! Domain size
      real         dx,dy                ! Horizontal resolution
      integer      nx,ny,nz             ! Grid dimensions
      real         pollon,pollat,polgam ! Longitude and latitude of pole
      real         z3(nx,ny,nz)         ! Staggered levels
      real         zb(nx,ny)            ! Surface pressure
      real         time                 ! Time of the grid information
      real         stagz                ! Vertical staggering (0 or -0.5)
      character*80 fieldname            ! Variable from which to take grid info
      character*80 timecheck            ! Either 'yes' or 'no'
      
c     Numerical and physical parameters
      real          eps                ! Numerical epsilon
      parameter    (eps=0.001)

c	  Name of constants file
      character*80 cstfile
      parameter    (cstfile='LMCONSTANTS')

c     Netcdf variables
      integer      vardim(4)
      real         varmin(4),varmax(4)
      real         mdv
      real         stag(4)
      integer      ndim
      integer      cstid
      real         times(10)
      integer      ntimes
      integer      nvars
      character*80 vars(200)

c     Auxiliary variables
      integer      ierr       
      integer      i,j,k
      integer      isok
      real         tmp(200)
      character*80 varname
      real         rtime
      real         tmp3(nx,ny,nz+1)
      character*80 tname
      character*80 zname
    
c     Special handling if field LMCONSTANTS is requested and fid < 0 - get all 
c     grid parameters from the file LMCONSTANTS:
      if ( (fieldname.eq.'LMCONSTANTS').and.(fid.lt.0) ) then
            call cdfopn(cstfile,cstid,ierr)
            varname = 'HHL'
            call getdef(cstid,varname,ndim,mdv,vardim,
     >                  varmin,varmax,stag,ierr)
            if (ierr.ne.0) goto 900
            if (ierr.ne.0) goto 903
            call getpole(cstid,pollon,pollat,polgam,ierr)
            if (ierr.ne.0) goto 903
            call clscdf(cstid,ierr)
            if (ierr.ne.0) goto 903
            
            nx   = vardim(1)
            ny   = vardim(2)
            nz   = vardim(3) - 1 
            xmin = varmin(1)
            ymin = varmin(2)
            xmax = varmax(1)
            ymax = varmax(2)
            dx   = (xmax-xmin)/real(nx-1)
            dy   = (ymax-ymin)/real(ny-1)

           if ( xmin.lt.-180.) then
             xmin = xmin + 360.
             xmax = xmax + 360.
           else if ( xmax.gt.360. ) then
             xmin = xmin - 360.
             xmax = xmax - 360.
           endif

           goto 100
      endif
    
c     Get the definition for the variable - remember staggering
      varname = fieldname
      call getdef(abs(fid),varname,ndim,mdv,vardim,
     >            varmin,varmax,stag,ierr)
      if (ierr.ne.0) goto 900
      stagz = stag(3)

c     Special treatment for 2D fields: set vardim(3)=nz, stagz=0
      if ( vardim(3).le.1 ) then
          vardim(3) = nz
          stagz     = 0.
      endif

c	  Force some hard tests for grid consistency (stagz=0 for U grid; stagz<>0 for W grid)
c     <nz> must be the dimension of the U grid; the W grid has <nz+1>. For the W grid, we
c     will keep only the lowest <nz> model levels.
      if ( fid.ge.0 ) then
        if ( abs(stagz).lt.eps ) then
            if ( vardim(3).ne.nz ) goto 907
        else
            if ( vardim(3).ne.(nz+1) ) goto 907
            vardim(3) = vardim(3) -1
        endif
      endif

c     Set the grid dimensions and constants
      nx   = vardim(1)
      ny   = vardim(2)
      nz   = vardim(3)
      xmin = varmin(1)
      ymin = varmin(2)
      xmax = varmax(1)
      ymax = varmax(2)
      dx   = (xmax-xmin)/real(nx-1)
      dy   = (ymax-ymin)/real(ny-1)

c     We want the longitudes within -180 ... + 180
      if ( xmin.lt.-180.) then
         xmin = xmin + 360.
         xmax = xmax + 360.
      else if ( xmax.gt.360. ) then
         xmin = xmin - 360.
         xmax = xmax - 360.
      endif

c     Get pole position
      call cdfopn(cstfile,cstid,ierr)
      if (ierr.ne.0) goto 903
      call getpole(cstid,pollon,pollat,polgam,ierr)
      if (ierr.ne.0) goto 903
      call clscdf(cstid,ierr)
      if (ierr.ne.0) goto 903

c     Do not read any data if fid<0 - only return dimensions and grid info
      if ( fid.lt.0 ) goto 100

c     Get time information (check if time is correct)
      call gettimes(fid,times,ntimes,ierr)
      if (ierr.ne.0) goto 901
      isok=0
      do i=1,ntimes
         if (abs(time-times(i)).lt.eps) then
            isok = 1
            rtime = times(i)
         elseif (timecheck.eq.'no') then
            isok = 1
            rtime = times(1)
         endif
      enddo
      if ( isok.eq.0) goto 905
      
c     Open the constants file
      call cdfopn(cstfile,cstid,ierr)
      if (ierr.ne.0) goto 903

c     Get a list of all variable names and get name of topography and model levels
      call getvars(cstid,nvars,vars,ierr)
      if (ierr.ne.0) goto 903
      tname='nil'
      do i=1,nvars
         if ( vars(i).eq.'HSURF' ) then
            if ( tname.eq.'nil' ) then
               tname = 'HSURF'
            else
               print*,' WARNING : multiple topogaphy fields ',
     >                trim(tname), ' / HSURF'
               tname = 'HSURF'
            endif
         endif
         if ( vars(i).eq.'ZB' ) then
            if ( tname.eq.'nil' ) then
               tname = 'ZB'
            else
               print*,' WARNING : multiple topogaphy fields ',
     >                trim(tname), ' / HSURF'
               tname = 'ZB'
            endif
         endif
      enddo
      
      zname='nil'
      do i=1,nvars
         if ( vars(i).eq.'HHL' ) then
            if ( zname.eq.'nil' ) then
               zname = 'HHL'
            else
               print*,' WARNING : multiple height fields ',
     >                trim(zname), ' / HHL'
               zname = 'HHL'
            endif
         endif
         if ( vars(i).eq.'Z' ) then
            if ( zname.eq.'nil' ) then
               zname = 'Z'
            else
               print*,' WARNING : multiple height fields ',
     >                trim(zname), ' / Z'
               zname = 'Z'
            endif
         endif
      enddo

      if ( (tname.eq.'nil').or.(zname.eq.'nil') ) then
         print*,' ERROR: topography and/or height field is missing'
         stop
      endif


c     Read topography
      call gettimes(cstid,times,ntimes,ierr)
      if (ierr.ne.0) goto 904
      varname = tname
      call getdat(cstid,varname,times(1),0,zb,ierr)
      if (ierr.ne.0) goto 904

c	  Read 3d model height - it must match with the W grid and must
c	  therefore have <nz+1> levels
      varname = zname
      call getdef(cstid,varname,ndim,mdv,vardim,
     >            varmin,varmax,stag,ierr)
      if (ierr.ne.0) goto 906
      if ( vardim(3).ne.(nz+1) ) goto 907
      call gettimes(cstid,times,ntimes,ierr)
      if (ierr.ne.0) goto 904
      call getdat(cstid,varname,times(1),0,tmp3,ierr)
      if (ierr.ne.0) goto 906
            
c     Select the correct data height depending on vertical staggering;
c	  For the U grid (stagz=0) we compute it from the W grid's HHL
      if ( abs(stagz).lt.eps ) then
         do i=1,nx
            do j=1,ny
               do k=1,nz
                  z3(i,j,k) = 0.5 * ( tmp3(i,j,k) + tmp3(i,j,k+1) )
               enddo
            enddo
         enddo
      else 
         do i=1,nx
            do j=1,ny
               do k=1,nz
                  z3(i,j,k) = tmp3(i,j,k)
               enddo
            enddo
         enddo
      endif

c     Close the constants file
      call clscdf(cstid,ierr)
      if (ierr.ne.0) goto 903
 
c     Exit point
 100  continue
      
c     Exception handling
      return
      
 900  print*,'Cannot retrieve grid dimension from ',fid
      stop
 901  print*,'Cannot retrieve grid parameters from ',fid
      stop
 903  print*,'Problem with level coefficients from ',trim(cstfile)
      stop
 904  print*,'Cannot read surface height from ',trim(cstfile)
      stop
 905  print*,'Cannot find time ',time,' on ',fid
      stop
 906  print*,'Cannot read 3D model levels ',fid
      stop
 907  print*,'Grid inconsistency - stagz=0 for U grid; <>0 for W grid'
      stop
      
      end

c     ------------------------------------------------------------
c     Read wind information
c     ------------------------------------------------------------

      subroutine input_wind (fid,fieldname,field,time,stagz,mdv,
     >                       xmin,xmax,ymin,ymax,dx,dy,nx,ny,nz,
     >                       timecheck)

c     Read the wind component <fieldname> from the file with identifier
c     <fid> and save it in the 3d array <field>. The vertical staggering 
c     information is provided in <stagz> and gives the reference to either
c     the layer or level field from <input_grid>. A consistency check is
c     performed to have an agreement with the grid specified by <xmin,xmax,
c     ymin,ymax,dx,dy,nx,ny,nz>.

      implicit none

c     Declaration of variables and parameters
      integer      fid                 ! File identifier
      character*80 fieldname           ! Name of the wind field
      integer      nx,ny,nz            ! Dimension of fields
      real         field(nx,ny,nz)     ! 3d wind field
      real         stagz               ! Staggering in the z direction
      real         mdv                 ! Missing data flag
      real         xmin,xmax,ymin,ymax ! Domain size
      real         dx,dy               ! Horizontal resolution
      real         time                ! Time
      character*80 timecheck           ! Either 'yes' or 'no'

c     Numerical and physical parameters
      real        eps                 ! Numerical epsilon
      parameter  (eps=0.001)
      real        notimecheck         ! 'Flag' for no time check
      parameter  (notimecheck=7.26537)

c     Netcdf variables
      integer      ierr
      character*80 varname
      integer      vardim(4)
      real         varmin(4),varmax(4)
      real         stag(4)
      integer      ndim
      real         times(10)
      integer      ntimes
      character*80 cstfile
      integer      cstid
      real         ps(nx,ny)

c     Auxiliary variables
      integer      isok
      integer      i,j,k
      integer      nz1
      real         rtime
      real         tmp3(nx,ny,nz+1)

c     Read variable definition
      varname = fieldname
      call getdef(fid,varname,ndim,mdv,vardim,
     >            varmin,varmax,stag,ierr)
      if (ierr.ne.0) goto 900
      stagz=stag(3)

c     Check that the staggering is what we expect - <stagz=0> for U grid
      if ( vardim(3).gt.1 ) then
        if ( abs(stagz).lt.eps) then
	       if ( vardim(3).ne.nz) goto 905
	    else
	       if ( vardim(3).ne.(nz+1)) goto 905
        endif
      endif
      
c     Get time information (set time to first one in the file)
      call gettimes(fid,times,ntimes,ierr)
      if (ierr.ne.0) goto 902
      isok=0
      do i=1,ntimes
         if (abs(time-times(i)).lt.eps) then
            isok = 1
            rtime = times(i)
         elseif (timecheck.eq.'no') then
            isok = 1
            rtime = times(1)
         endif
      enddo
      if ( isok.eq.0 ) goto 904

c     Read the data - correctly handling of vertical staggering
      if ( abs(stagz).lt.eps ) then
         varname=fieldname
         call getdat(fid,varname,rtime,0,field,ierr)
         if (ierr.ne.0) goto 903

      else
         varname = fieldname
         call getdat(fid,varname,rtime,0,tmp3,ierr)
         if (ierr.ne.0) goto 903
         do i=1,nx
            do j=1,ny
               do k=1,nz
                  field(i,j,k) = tmp3(i,j,k)
               enddo
            enddo
         enddo
      endif

c     If the field is 2D, expand it to 3D - simple handling of 2D tracing
      if ( vardim(3).le.1 ) then
         do i=1,nx
            do j=1,ny
               do k=1,nz
                  field(i,j,k) = field(i,j,1)
               enddo
            enddo
         enddo
      endif
         


c     Exception handling
      return
      
 900  print*,'Cannot retrieve definition for ',trim(varname),'  ',fid
      stop
 901  print*,'Grid inconsistency detected for ',trim(varname),'  ',fid
      stop
 902  print*,'Cannot retrieve time for ',trim(varname),'  ',fid
      stop
 903  print*,'Cannot load wind component ',trim(varname),'  ',fid
      stop
 904  print*,'Cannot load time ',time,' for ',trim(varname),'  ',fid
      stop
 905  print*,'grid inconsistency - stagz=0 for U grid; <>0 for W grid'
      stop
     
      end

c     ------------------------------------------------------------
c     Close input file
c     ------------------------------------------------------------

      subroutine input_close(fid)

c     Close the input file with file identifier <fid>.

      implicit none

c     Declaration of subroutine parameters
      integer fid

c     Auxiliary variables
      integer ierr

c     Close file
      call clscdf(fid,ierr)
 
      end
